/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   SCEO_tls - Modulo TLS para o servidor MTA Sceo                      
   Programado por Lucas Priori                                         
   Versao: 1.0                                                         
                                                                       
   Para Compilar:                                                      
   Linux - gcc -shared -nostartfiles sceo_tls.c -o sceo_tls.so -lssl -fPIC
   FreeBSD - gcc -shared -nostartfiles sceo_tls.c -o sceo_tls.so -lssl -fPIC
                                                                       
  *******************************************************************  
  O sceo_tls.so  um modulo que habilita o recurso de criptografia TLS 
  ao servidor MTA Sceo                                                 
  Copyright (C) 2012  Lucas Priori                                     
                                                                       
  This program is free software: you can redistribute it and/or modify 
  it under the terms of the GNU General Public License as published by 
  the Free Software Foundation, either version 3 of the License, or    
  (at your option) any later version.                                  
                                                                       
  This program is distributed in the hope that it will be useful,      
  but WITHOUT ANY WARRANTY; without even the implied warranty of       
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
  GNU General Public License for more details.                         
                                                                       
  You should have received a copy of the GNU General Public License    
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#include<stdio.h>
#include<sys/socket.h>
#include<netinet/in.h>
#include<openssl/ssl.h>
#include<openssl/bio.h>

#define TAM_buffer 1000

// --- Variaveis Globais ---
SSL *ssl;
BIO *bio_stderr, *bio_socket;
SSL_CTX *ctx;
char buffer_error[TAM_buffer];

void _init(){
 return;
}
void _fini(){
 return;
}

//=======================================================
// Funcao para caregar e preparar o SSL e seus arquivos  
// Nao  usada neste modulo. Sao carregas pela tls_init  
// Retornos:                                             
// 0 = Sucesso                                           
// -1 = Erro                                             
char tls_load(char *mod_params, char *ca_file, char *private_key, char *public_key){
 
 SSL_library_init();
 SSL_load_error_strings();
 bio_stderr=BIO_new_fp(stderr, BIO_NOCLOSE);

 return 0;
}

//=======================================================
// Funcao para iniciar a conexao TLS no socket passado   
// Retornos:                                             
// 0 = Sucesso                                           
// -1 = Erro                                             
char tls_init(char *mod_params, int socket_tls, char *ca_file, char *private_key, char *public_key){
 
 if(!(ctx=SSL_CTX_new(SSLv23_server_method())))return -1;
 if(!(ssl=SSL_new(ctx)))return -1;
 if(!(SSL_use_certificate_file(ssl, public_key, SSL_FILETYPE_PEM)))return -1;
 if(!(SSL_use_PrivateKey_file(ssl, private_key, SSL_FILETYPE_PEM)))return -1;
 if(ca_file && ca_file[0])if(SSL_CTX_load_verify_locations(ctx, ca_file, NULL) != 1)return -1;
 SSL_set_fd(ssl, socket_tls);

 return 0;
}

//=======================================================
// Funcao para realiar o handshake TLS em modo CLIENTE   
// 0 = Sucesso                                           
// -1 = Erro                                             
char tls_connect(int socket_tls){

 if(!(ctx=SSL_CTX_new(SSLv23_client_method())))return -1;
 ssl=SSL_new(ctx);
 SSL_set_fd(ssl, socket_tls);
 if(SSL_connect(ssl) <= 0)return -1;
 return 0;
}


//=======================================================
// Funcao para realiar o handshake TLS em modo SERVIDOR  
char tls_accept(void){
 
 return SSL_accept(ssl);
}

//=======================================================
// Funcao para fechar a parte TLS do socket alocado      
char tls_close(void){

 //ERR_free_strings();
 SSL_shutdown(ssl);
 SSL_free(ssl);
 SSL_CTX_free(ctx);
 return 0;
}


//=======================================================
// Funcao para enviar um pacote via SOCKET TLS           
int tls_send(char *buffer_local, int tamanho){

 return SSL_write(ssl ,buffer_local, tamanho);
}

//=======================================================
// Funcao para receber um pacote via SOCKET TLS          
int tls_recv(char *buffer_local, int tamanho){

 return SSL_read(ssl ,buffer_local, tamanho);
}

//=======================================================
char *tls_error(void){
 
 ERR_error_string_n(ERR_get_error(), buffer_error, TAM_buffer);
 return buffer_error;
}


